# -*- coding: utf-8 -*-

from ..core.lang import add_lang

add_lang('en', {
    'buy': 'buy',
    'sell': 'sell',
    'stop loss': 'stop loss',
    'stop earn': 'stop earn',
    'signal': 'signal',
})
